#!/usr/bin/python
# -*- coding: utf8 -*-
import os
import io;
import sys
import types
import sqlite3
import tempfile

from string import *
from ctypes import *

CN_PAKET_SIZE   = 0xA00000; #10M


#===============================================================
# partiton handle
class Partion:
  def __init__(self, label, fstype, size, flag, downloadfile):
    self.label = label
    self.fstype = fstype
    self.size = size
    self.flag = flag
    self.downloadfile = downloadfile


def	CheckKey(line, keyword):
    line_list = [];
    if line.count(keyword)==True :
        line_list = line.split('=');
        if line_list[0].strip() == keyword  :
            return line_list[1].strip();
    return None;
 
def	Build_Partion(str_cfg, partion_list):
    
    bpart = False;
    label = None;
    fstype = None;
    size = None;
    flag = None;
    downloadfile = None;
    line_list = [];
    line_list = str_cfg.split('\n');

    for line in line_list:
        if line.strip() == '' or line[0] in '#;//':
            continue
        if line.count('[MBR_INFO]')==True :
            bpart = True;
            label = 'MBR';
            continue;
        if line.count('[partition]')==True :
            if bpart == True :
                partion_list.append( Partion(label, fstype, size, flag, downloadfile) );
            bpart = True;
            label = None;
            fstype = None;
            size = None;     
            flag = None;
            downloadfile = None;
            continue;
            
        if bpart != True :
            continue;

        tmp = CheckKey(line,'label');
        if tmp != None:
            label = tmp;
            continue;
        tmp = CheckKey(line,'fstype');
        if tmp != None:
            fstype = tmp;
            continue;           
        tmp = CheckKey(line,'size');
        if tmp != None:
            size = tmp;
            continue;
        tmp = CheckKey(line,'flag');
        if tmp != None:
            flag = tmp;
            continue;
        tmp = CheckKey(line,'downloadfile');
        if tmp != None:
            downloadfile = tmp;
            continue; 

    if label != None :
        partion_list.append( Partion(label, fstype, size, flag, downloadfile) );



#===============================================================
# Redirect the output

class stdout:
    def write(self, buf):
        if ((len(buf) > 0) & (buf[0] != '\n')):
            windll.kernel32.OutputDebugStringA(buf);

def memcpy(cbuffer, pylist, func):
    size = min(sizeof(cbuffer), len(pylist));
    for i in range(size):
        cbuffer[i]= func(pylist[i]);


def file_8_3(fname):
    pos = fname.index('.');
    if (pos < 0 or pos > 8):
        return None; #file name more than 8
    if (len(fname) - pos - 1 > 3):
        return None; #file extension more than 3
    fname = fname.upper();
    ext = len(fname) - pos - 1;
    padding = '        ';
    return fname[0:pos] + padding[0:8-pos] + fname[pos+1:] + padding[0:3-ext];

def checksum_2(cbuffer, size):
    ret = c_uint16(0);
    ptr = cast(cbuffer, POINTER(c_uint16));
    size= size / 2;
    for i in range(size):
        ret.value = ret.value + ptr[i];
    return ret;

def checksum_4(cbuffer, size):
    ret = c_uint32(0);
    ptr = cast(cbuffer, POINTER(c_uint32));
    size= size / 4;
    for i in range(size):
        ret.value = ret.value + ptr[i];
    return ret;  


#===============================================================
# Firmware Scripter

class FirmwareScript:
    r"""Firmware script
    """
    def __init__(self):
        self.lib = None;
        self.sql_conn  = None;
        self.image = c_buffer('\0', 520);
        self.tmp_dir = '';

    def __del__(self):
        r"""Destruction
        """
        self.Close();

    def Load(self, fw, lib):
        r"""Load firmware file and libScript.so
        Connect firmware database as memory
        return true if succeeded
        """
        if (True != os.path.isfile(lib)):
            raise Exception('%s not exsit!' %lib);

        if (True != os.path.isfile(fw)):
            raise Exception ('Firmware (%s) not exsit!' %fw);

        fwBin = None;
        fwLen = 0;
        try:
            self.lib = cdll.LoadLibrary(lib);
            fwBin = c_buffer('\0', 0x100000); # 1MB for temp.db
            fwLen = self.lib.Py_OpenFirmware(fw, fwBin, self.image);
        except Exception as e:            
            if (fwBin):
                del fwBin;
            raise Exception('Exception occurred in FirmwareScript.Load: %s(%s)' %(type(e),e));    

        if (fwLen == 0):            
            del fwBin;
            raise Exception('Read firmware %s failed!' %fw);

        # temporary file
        self.tmp_dir = '';
        path = str(self.image.value).split(str(os.sep));
        for i in range(len(path)-1):
            self.tmp_dir += path[i];
            self.tmp_dir += str(os.sep);
        #print self.image.value;

        # libFileSystem.so
        if (os.name == 'posix'):  
            fs_dir = '';
            path = lib.split(str(os.sep));
            for i in range(len(path)-1):
                fs_dir += path[i];
                fs_dir += str(os.sep);
            #print self.image.value;
            self.lib.Py_SetLibPath(fs_dir);
   
        # Mapped as memory database        
        tmp = tempfile.NamedTemporaryFile(delete=False, dir=self.tmp_dir);
        tmp.write(fwBin);
        del fwBin;
        tmp.close();

        self.sql_conn = sqlite3.connect(":memory:");
        self.sql_conn.isolation_level = None;
        
        try:
            tmpDB = sqlite3.connect(tmp.name);
            for line in tmpDB.iterdump(): 
                self.sql_conn.execute(line);            
            tmpDB.close();
        except sqlite3.Error as e:
            
            os.remove(tmp.name);
            self.sql_conn = None;
            raise Exception('Exception occurred when connect database 87: %s' %e.args[0]);
    
        os.remove(tmp.name);
        
        return True;

    def Open(self, fwBin, fwImage, lib):
        r"""Load binary firmware database and connect it as memory
        Specify the temp.img and libScript.so
        return true if succeeded
        """
        if (True != os.path.isfile(lib)):
            raise Exception('%s not exsit!' %lib);

        if (fwBin == None):
            raise Exception('Read firmware %s failed!' %fw);

        try:
            self.lib = cdll.LoadLibrary(lib);
        except Exception as e:
            raise Exception('Exception occurred in FirmwareScript.Open: %s(%s)' %(type(e),e));    

        self.image = fwImage;
        
        # temporary file
        self.tmp_dir = '';
        path = str(self.image.value).split(str(os.sep));
        for i in range(len(path)-1):
            self.tmp_dir += path[i];
            self.tmp_dir += str(os.sep);

        # libFileSystem.so
        if (os.name == 'posix'):  
            fs_dir = '';
            path = lib.split(str(os.sep));
            for i in range(len(path)-1):
                fs_dir += path[i];
                fs_dir += str(os.sep);
            #print self.image.value;
            self.lib.Py_SetLibPath(fs_dir);
   
        # Mapped as memory database        
        tmp = tempfile.NamedTemporaryFile(delete=False, dir=self.tmp_dir);
        tmp.write(fwBin);
        tmp.close();
        # print "%s\n%s"%(self.image.value, tmp.name);	

        self.sql_conn = sqlite3.connect(":memory:");
        self.sql_conn.isolation_level = None;
       
        try:
            tmpDB = sqlite3.connect(tmp.name);
            for line in tmpDB.iterdump(): 
                self.sql_conn.execute(line);            
            tmpDB.close();
        except sqlite3.Error as e:
            os.remove(tmp.name);
            self.sql_conn = None;
            raise Exception('Exception occurred when connect database 136: %s' %e.args[0]);
    
        os.remove(tmp.name);
        
        return True;

    def IsLoaded(self):
        if (os.path.isfile(self.image.value) != True):
            return False;
        
        if (self.sql_conn == None):
            return False;
        
        return True;

    def Close(self):
        r"""Close firmware sqlite connection
        """            
        if (self.sql_conn != None):
            self.sql_conn.close();
            self.sql_conn = None;
            
    def Count(self, keyword):
        r"""Query keyword config count
        """
        if (self.sql_conn == None):
            return 0;
            
        c = self.sql_conn.cursor();
        c.execute('select count(*) from %s;' %keyword);
        row = c.fetchone();
        #print 'Count(%s) = %d' %(keyword, row[0]);
        return row[0];    

    def ParamKey(self, keyword, index):
        r"""Query value about config keyword by index
        """
        if (self.sql_conn == None):
            return None;
        
        c = self.sql_conn.cursor();
        c.execute('select * from %s order by 1;'% keyword);
        row = c.fetchone();
        if (row == None):
            return None;
        
        return row[index];

    def ParamParam(self, keyword, ref_val, ref_index, get_index):
        r"""Query value by keyword and reference value
        """
        if (self.sql_conn == None):
            return None;
        
        if (ref_index < 0):
            return None;

        c = self.sql_conn.cursor();
        c.execute('select * from %s order by 1;'% keyword);
        row = c.fetchall();

        if (row == None):
            return None;

        retList = [];
        for it in row:
            if (ref_index >= len(it)):
                break;
        
            if (it[ref_index] != ref_val):
                continue;
            
            for g in get_index:
                retList.append(it[g]);   
            
        return retList;

    def QueryAll(self, keyword):
        r"""Query value by keyword and return all record rows 
        """
        if (self.sql_conn == None):
            return None;
        
        c = self.sql_conn.cursor();
        try:
            c.execute('select * from %s;'%(keyword));
            row = c.fetchall();
            return row;
        except Exception as e:
            print 'Exception occurred in QueryAll(%s): %s(%s)' %(keyword, type(e),e);

        return None;        

    def FileSize(self, fname, align = 0):
        r"""Query file size by specified name
        """
        if (self.sql_conn == None):
            return 0;
        
        c = self.sql_conn.cursor();

        if (fname != None): # one file only
            c.execute('select FileLength from FileTable where FileName = \'%s\';'%fname);
            row = c.fetchone();
            if (row != None):
                return int(row[0]);
            
        else: # sum all file length
            c.execute('select FileLength from FileTable;');
            row = c.fetchall();
            size = 0;
            for i in row:
                size = size + (int(i[0]) + align) / align * align;
            return size;
        
        return 0;

    def FileRead(self, fname, offset, align, buf, myThreadLock = None):
        r"""Read internal binary file from firmware
        """
        if (self.lib == None):
            return 0;

        if (os.path.isfile(self.image.value) != True):
            raise Exception('temp.img non-exsitent!');
        
        if (None != myThreadLock):
            myThreadLock.acquire(); #Get the lock

        read = 0;
        
        try:
            read = self.lib.Py_ReadFileInFW(self.image, fname.encode('utf-8'), offset, len(buf), buf, align);
        except Exception as e:
            read = 0;
            raise Exception('Exception occurred in FileRead: %s(%s)' %(type(e),e));
        
        if (None != myThreadLock):
            myThreadLock.release();
        
        return read;
    
    def FileWrite(self, fname, buf, length, myThreadLock = None):
        r"""Read internal binary file from firmware
        """
        if (self.lib == None):
            return 0;

        if (os.path.isfile(self.image.value) != True):
            raise Exception('temp.img non-exsitent!');
        
        if (None != myThreadLock):
            myThreadLock.acquire(); #Get the lock

        read = 0;
        
        try:
            if (self.sql_conn != None):
                c = self.sql_conn.cursor();
                c.execute('INSERT INTO FileTable VALUES(\'TMP_BUILD\', \'%s\', %d, \'\');'%(fname, length));
    
            read = self.lib.Py_AddFileToFW(self.image, fname.encode('utf-8'), buf, length);
        except Exception as e:
            read = 0;
            raise Exception('Exception occurred in FileWrite: %s(%s)' %(type(e),e));
        
        if (None != myThreadLock):
            myThreadLock.release();
        
        return read;

    def memcmp(self, p1, p2, size, ignore_case):
        """
        memory compare, byte unit
        @lib is the c-library instance which export the memcmp api
        @ignore_case: 0 for case-sensitive
        """
        return self.lib.Py_MemCmp(p1, p2, size, ignore_case);
    
#############################################################
### Unit test code
#############################################################

if __name__ == '__main__':
    #lib = sys.argv[1];#'/home/iter/705a/bin/12.04/x64/libScript.so';
    #fw = sys.argv[2];#'/home/iter/705a/firmware/lollipop_atm7059a_demo_86v_150107.fw';  
    if (os.name == 'posix'):
        lib = '/home/iter/705a/bin/12.04/x64/libScript.so';
        fw = '/home/iter/firmware/lollipop_atm7059a_demo_86v_150107.fw';    
    elif (os.name == 'nt'):
        lib = 'D:\\Code\\Production Tool - SVN5470 - PAD_BOX\\Bin\\Script.dll';
        fw = 'H:\\Firmware\\705A\\lollipop_atm7059a_demo_86v_150107.fw';  

    fws = FirmwareScript();
    try:
        fws.Load(fw, lib);
        fn = fws.ParamKey("ADFUS", 0);
        print 'ADFUS: %s'%(fn);
        print '%s size: %d' %(fn, fws.FileSize(fn));
        adfus = c_buffer('\0', 0xA00000);
        print 'FileRead: %d' %(fws.FileRead(fn, 0, 512, adfus, None));
        print fws.memcmp('abcd', 'ABCD', 4, True);
        fws.Close();
    except Exception as e:
        print '%s(%s)' %(type(e),e);    




