#!/usr/bin/python
# -*- coding: utf-8 -*-
import os
import sys
import thread
import re
import mmap
import struct
import StringIO
import codecs
import math
import ProductionV2

from CommonV2 import *;

info="--please input 'sudo python./ActionsFWU.py --help',view help!"
errorInfo = '--Incorrect command input!'

#===============================================================
# python thread, for multiple devices simultaneously on windows 
def ProductionThread(script_dll, product_dll, product_tid, app_type = 1):
    """
    工具调用的入口
    app_type: 指定当前应用程序执行的类型:
        0x01:ADFU量产 / 0x02:ADFU升级 / 0x03:U盘量产 / 0x04:U盘升级 / 0x05:FLASH_DUMP
    """
    import ProductionV2;
    return thread.start_new_thread(ProductionV2.Production, (script_dll, product_dll, product_tid, app_type, ''));

#===============================================================
# python thread
# the making process takes a long time, so run it in thread
def CardThread(script, maker, disk):
    import MakeCardV2;
    return thread.start_new_thread(MakeCardV2.MakeCard, (script, maker, disk, ''));

def MakeBin(script, fw, argv, fname):
    """
    根据固件配置提取文件
    script： 指定script.dll，同量产
    fw: 指定固件路径，temp.db + temp.img形式
    argv: 要提取的对象，比如：AFI
    fname： 目标路径，用来保存提取结果
    """
    import ProductionV2;
    return ProductionV2.MakeBin(script, fw, argv, fname);

def usage():
    print 'Usage: sudo ./ActionsFWU [OPTION]...'
    print 'Optional Features:'
    print '  --help         display this help and exit.'
    print '  --fw=FIRMWARE  then firmware file will be burned into device, absolute path.'
    print '  --list         display all partitions in the firmware and exit.'
    print '  --all          burn all partitions in the firmware, default option.'
    print '  --erase        erase boot partition and exit,' 
    print '                 pN:Default is not erase unless use the option --erase '
    print '  --pN[=IMG]     N=[0-16], burn the partition N only with IMG file.'
    print '                 if the partition is FMT type,the IMG argument is ignored.'
    print 'NOTE:'
    print '  Please note that using lower case letter writing instruction.'
    print '  you can use the option all or pN to start burning procedure,'
    print '  but the option al an pN can`t be used at the same time.'
    print '  you can use more than one option pN to burn more partitions at the same time'

#===================================================================================
def parse_help(argv):
    info="Instruction is error,please input 'sudo ./ActionsFWU.py --help',look at the help"
    
    if(argv.count('--help') == True):
        paraHelp = argv.split('help');
        if(paraHelp[0] != '--'):
            return False;
        paraHelp= argv.split('--');
        if(paraHelp[1] != 'help'):
            return False;
        return True;  
    else:
        return False;
#===================================================================================
def parse_erase(argv):
    if(argv.count('--erase') == True):
        paraErase = argv.split('erase');
        if(paraErase[0] != '--'):
            return False;
        paraErase= argv.split('--');
        if(paraErase[1] != 'erase'):
            return False;
        return True;  
    else:
        return False;
#===================================================================================
def parse_list(argv):
    if(argv.count('--list') == True):
        paraList = argv.split('list'); 
        if(paraList[0] != '--'):
            return False;
        paraList= argv.split('--');
        if(paraList[1] != 'list'):
            return False;
        return True;  
    else:
        return False;
#===================================================================================
def parse_all(argv):
    if(argv.count('--all') == True):
        # String by 'all' division
        paraAll = argv.split('all');
        if(paraAll[0]!='--'):
            return False;
        paraAll= argv.split('--');
        #Judge whether the character 'all' lowercase
        if(paraAll[1]!='all'):
            return False;
        return True;
    else:
        return False;
#===================================================================================
def parse_fw(argv):
    if(argv.count('--fw=') == True):
        # String by 'fw=' division
        paraFw = argv.split('fw=');
        if(paraFw[0] != '--'):
            return False;
        paraFw= argv.split('=');
        if(paraFw[0] != '--fw'):
            return False;
        if(argv.find('.fw')<0):
            return False;
        para.fw=paraFw[1];
        return True;  
    else:
        return False;
#===================================================================================
def parse_devmode(argv):
    if(argv.count('--devmode') == True):
        paraDevmode = argv.split('devmode');
        if(paraDevmode[0] != '--'):
            return False;
        if(argv.count('=') == False):
            para.devmode = True;
            #set devmode:adfu
            para.devmodepara = '256';
            return True;
        paraDevmode = argv.split('=');
        if(paraDevmode[1]=='1'):
            #set devmode:udisk
            para.devmode = True;
            para.devmodepara = '1';
            return True;
        elif(paraDevmode[1]=='256'):
            #set devmode:adfu
            para.devmode = True;
            para.devmodepara = '256';
            return True;
        elif(paraDevmode[1]=='257'):
            #set devmode:udisk adfu
            para.devmode = True;
            para.devmodepara = '257';
            return True;
    else:
        return False;
    
#===================================================================================
def read_int(stream):
    result = struct.unpack("i", stream.read(4))
    #print type(result)
    return result[0]
#===================================================================================
def read_chars(reader,cnt):
    s = reader.read(chars=cnt, size=1)
    return s
     
#===================================================================================
def read_data_from_img(dataPath,flag):
    dataFile = dataPath.split('='); 
    fileName=dataFile[len(dataFile)-1];
    with codecs.open(fileName, "r", "utf8", "ignore") as f:
        stream = f.stream
        count = read_int(stream)
        pos = f.tell()
        for index in xrange(4):
            market = read_int(stream)
            quote_type = read_int(stream)        
            code = read_chars(f.reader,6)
            stockname = read_chars(f.reader,16)
            pinyin =  read_chars(f.reader,8)
            lotsize = read_int(stream)
            underlying = read_int(stream)
            exdate = read_int(stream)
            stock = 'market:%s,type:%s,stockname:%s,lotsize:%s' % (market,quote_type,stockname,lotsize)
            unicode_to_str=stockname.encode("utf-8");
            imgName=unicode_to_str.split();
            #转化大写，进行类别辨析
            if(imgName[0].upper() == flag and imgName[1] == 'FAT32'):
                print imgName;
                return True;
#            elif(imgName[0].upper() == flag and imgName[1] == 'FAT32'):
#                print imgName;
#                return True;
    return False;
#===================================================================================
def parse_partion_para3_n(argv,length):
    flag='';
    paraName=argv.split('=');
    pn='p'+paraName[0][3:];
    paraName=argv.split('/'); 
       
    for i in range(3,length):
        pN='p'+str(i);     
        if(pn==pN and para.partition_cfg[i][0]==pN):
            if(para.partition_cfg[i][1]=='RECOVERY'):
                if(paraName[len(paraName)-1]!='recovery.img'):
                    print "please download recovery.img in %s"%(pN)
                    return False;
            elif(para.partition_cfg[i][1]=='SYSTEM'):
                if(paraName[len(paraName)-1]!='system.img'):
                    print "please download system.img in %s"%(pN)
                    return False;
            elif(para.partition_cfg[i][1]=='BOOT_MSG'):
                print "%s is format partition,Don't download image!"%pN;
                return False;
            elif(para.partition_cfg[i][1]=='DATA'):
                print "%s is format partition,Don't download image!"%pN;
                return False;
            elif(para.partition_cfg[i][1]=='CACHE'):
                print "%s is format partition,Don't download image!"%pN;
                return False;
            elif(para.partition_cfg[i][1]=='SWAP'):
                print "%s is format partition,Don't download image!"%pN;
                return False;
            elif(para.partition_cfg[i][1]=='VENDOR_APP'):
                flag='VENDOR_APP';
                if(argv.find('.img')<0):
                    print "the file format is error in the VENDOR_APP partition";
                    return False;
                elif(read_data_from_img(argv,flag)==False):
                    print "Don't download the file in the VENDOR_APP partition!"
                    print "Please download the file that the Label is 'VENDOR_APP'"
                    return False;
            elif(para.partition_cfg[i][1]=='MNT_MEDIA'):
                flag='MNT_MEDIA';
                if(argv.find('.img')<0):
                    print "the file format is error in the MNT_MEDIA partition"
                    return False;
                elif(read_data_from_img(argv,flag)==False):
                    print "Don't download the file in the MNT_MEDIA partition!"
                    print "Please download the file that the Label is 'MNT_MEDIA'"
                    return False;
            elif(para.partition_cfg[i][1]=='UDISK'):
                if((paraName[len(paraName)-1]=='misc.img') or \
                   (paraName[len(paraName)-1]=='recovery.img') or \
                   (paraName[len(paraName)-1]=='system.img')):
                    print "Don't download the %s file in the UDISK partition!"%(paraName[len(paraName)-1]);
                    return False;

    return True;

#===================================================================================
def parse_partion_para0_2(argv,length,pn):
    paraName=argv.split('/');
    if(pn=='p0'):
        if(paraName[len(paraName)-1] != 'bootloader.bin'):
            return False;
    if(pn=='p1'):
        if(paraName[len(paraName)-1].count('uboot') == False):
            if(paraName[len(paraName)-1].count('u-boot') == True):
                return True;
            else:
                return False;
        else:
            return True;
    if(pn=='p2'):
        if(paraName[len(paraName)-1] != 'misc.img'):
            return False;
    if(parse_partion_para3_n(argv,length) == False):
        return False;
    return True;


#===================================================================================
def parse_other(argv,length):
    errInfo='invalid literal'
    paraName = [];

    try:
        paraName = argv.split('p');
        #judge '--'
        if(paraName[0] != '--'):
            return False;
        paraName = argv.split('=') 
        if(int(paraName[0][3:])>16):
            return False;
        pn='p'+paraName[0][3:];
        p='--'+pn+'=/';
        if(re.search(p,argv) == None):
            return False
        #Judge file format ''.img' '.Bin
        if(argv.find('.') < 0):
            return False;
        if(parse_partion_para0_2(argv,length,pn)==False):
            return False;
        return True;
    except Exception as errInfo:
        print errInfo;

#===================================================================================
def parse_p(argv,length):    
    # String by 'p' division
    para_P = argv.split('p');
    #judge '--'
    if(para_P[0]!='--'):
        return False;
    #String by '--' division
    para_P= argv.split('--');
    #Judge whether the character 'p' lowercase
    if(para_P[1][0]!='p'):
        return False;
    #Judge whether the number is greater than 16
    if(int(para_P[1][1:])>16):
        return False;
#    para_P=argv.split('=');
#    pn='p'+para_P[0][3:];
#    for i in range(0,length):
#        pN='p'+str(i);     
#        if(pn==pN and para.partition_cfg[i][0]==pN): 
#            if(para.partition_cfg[i][1]=='VENDOR_APP'):
#                print "%s VENDOR_APP need the file that is downloaded!"%pN;
#                return False;
#            elif(para.partition_cfg[i][1]=='MNT_MEDIA'):
#                print "%s MNT_MEDIA need file that is downloaded!"%pN;
#                return False;
#            elif(para.partition_cfg[i][1]=='DATA_BAK'):
#                print "%s DATA_BAK need file that is downloaded!"%pN;
#                return False; 
#            elif(para.partition_cfg[i][1]=='UDISK'):
#                udiskTag=pN;
    return True;  

#===================================================================================
def Parse_pn(argv,path):
    errInfo = 'Instruction input error'
    ret = False;
    global partition_len;
    #indicate the udiskTag wether is taken 
    global udiskTag;
#    udiskTag=None;
    #list partition
    if(argv.count('--p') == True):
        para.list = True;
        if(para.parse_partition_tag == True):
            ProductionV2.Cmd_Production(path + 'libScript.so', path + 'libProduction.so', para.fw, para);
            partition_len = 0;
            #检查分区表的个数，para.partition_cfg[i][0]无分区
            for i in range(17):
                if(para.partition_cfg[i][0]!=0):
                    partition_len=partition_len+1;
                    ret =True;
                else:
                    break;
            para.parse_partition_tag = False;
        if(partition_len == 0):
            print 'NO partition!';
            ret = False;
            return ret;
        #judge '--'
        para_P = argv.split('p');
        if(para_P[0]!='--'):
            return False;

        para_P = argv.split('-');
        if(para_P[len(para_P)-1]=='p'):
            return False;

        line_list=argv.split('=');
        pn='p'+line_list[0][3:];
        #提取--P后面的数字，如p7 提取出为'7'
        if(int(line_list[0][3:]) >= partition_len and partition_len != 0):
            print '%s is more larger than the number of the partition'%(pn);
            ret = False;
            return ret;
    else:
        return False;
    if(ret == True):
        print 'the number of the partition:%d'%partition_len;
    #parse p
    try:
        if(argv.count('--p') == True):
            if(argv.count('=')==True):
                #检测下载指定路径文件
                ret = parse_other(argv,partition_len);
                return ret;
            else:
                ret =parse_p(argv,partition_len);
                return ret;
        else:
            return False;
    except Exception as errInfo:
        print errInfo;

#===================================================================================
def Parse_args(argv,path):
    ret = True;
    #parse --help --fw=
    for i in range(1,len(argv)):
        if(parse_help(argv[i])==True):
            usage();
            return True
        else:
            if(parse_fw(argv[1])==False):
                ret=False;
                break;
    #parse --all --list --erase --devmode 
    if(ret == True and len(argv)==3):
        if(parse_erase(argv[2])==True):
            return True;
        elif(parse_list(argv[2])==True):
            return True;
        elif(parse_all(argv[2])==True):
            return True;
        elif(parse_devmode(argv[2]) == True):
            return True;
        else:
            ret = False;
    elif(len(argv)==3):
        ret = False;
    #parse --p
    para.parse_partition_tag = True;
    for i in range(2,len(argv)):
        if(Parse_pn(argv[i],path)==True):
            ret = True;
            continue;
        else:
            ret = False;
            break;
    para.parse_partition_tag = False;
    #deal error 
    if(ret == False):
        print errorInfo;
        print info;
    return ret;

def Handle_Arg(argv, para):
    fw_file = None;
    blist = False;
    berase = False;
    part = {};
    for i in range(1, len(argv)):
        line_list = [];
        if (argv[i].count('--p') == True) :
            line_list = argv[i].split('=');
            if ( len(line_list) > 1) :
                pn = int(line_list[0][3:]);
                part[line_list[0][2:]] = line_list[1];
            else :
                part[line_list[0][2:]] = '\n';
        elif (argv[i].count('--fw') == True):
            line_list = argv[i].split('=');
            if (len(line_list) > 1) :
                fw_file = line_list[1];
            else :
                usage();
                return False;
        elif (argv[i].count('--list') == True):
                blist = True;
        elif (argv[i].count('--erase') == True):
                berase = True;
    partstr='';
    for i in range(16):
        key = 'p%d'%i;
        if part.has_key(key) == True :
            if part[key] == '\n' :
                partstr = partstr + key + part[key];
            else :
                partstr =  partstr + key + '=' + part[key]+'\n'; 

    if fw_file == None:
        print "--fw is not exist";
        usage();
        return False;
    para.fw = fw_file;

    if  blist == True :
        print 'display all partitions in the firmware';
        para.list = True;
        return True;

    if  berase == True :  
        print "=====erase boot partition====";
        para.erase = True;
        return True;
    
    if(para.devmode ==True):
        return True;

    para.all = True;
    if ( partstr != '' ) :
        print "=====burn some partition====";
        para.part = partstr;
        para.erase = False;
    else :
        print "=====burn all partition====";
        para.part = None;
        para.erase = True;
   
    return True;
    
#===================================================================================
def Execute(argv):
    """
    argv[0]: Launch.py的完整路径，和所依赖的so库在同一目录下
    argv[1]: 固件的完整路径
    argv[2..]: Launch.py的执行选项
    """
    global para;
    path= '';
    
    if(len(sys.argv)==1):
        print info;
        return False;
    dirs = str(sys.argv[0]).split(os.sep);
    for i in range(len(dirs)-1):
        path = path + dirs[i] + os.sep;
    para = ProductionV2.BurnPara();
    if(Parse_args(sys.argv,path)==True):
        para.list = False;
        if(sys.argv[1].count('--help')):
            return False;
        elif(parse_fw(sys.argv[1])==True and len(sys.argv)>2):
            if(sys.argv[2].count('help')):
                return False;
        if (Handle_Arg(sys.argv, para) == True) :
#            print 'path:' + para.fw;
            ProductionV2.Cmd_Production(path + 'libScript.so', path + 'libProduction.so', para.fw, para);

    #import ProductionV2;
    #ProductionV2.Production(path + 'libScript.so', path + 'libProduction.so', 0, 1, argv[1]);

#===================================================================================
if __name__ == '__main__':
    #Execute(argv);
    #dbg_fw = sys.argv[1];#'H:\\Firmware\\705A\\lollipop_atm7059a_demo_86v_150107_large.fw';
    #script = sys.argv[2];#'/home/iter/705a/firmware/lollipop_atm7059a_demo_86v_150107.fw';  
    #product= sys.argv[3];#'/home/iter/705a/bin/12.04/x64/libScript.so';
    print 'start burn, os name %s'%os.name;
    if (os.name == 'posix'):
        script = '/home/iter/705a/bin/12.04/x64/libScript.so';
        product= '/home/iter/705a/bin/12.04/x64/libProduction.so';
        dbg_fw = '/home/iter/firmware/lollipop_atm7059a_demo_86v_150120_large_udisk.fw';    
    elif (os.name == 'nt'):
        #product= 'D:\\Code\\Production Tool - SVN5470 - PAD_BOX\\Bin\\Production.dll';
        #product= 'C:\\Program Files\\Actions\\PAD量产工具\\2.00.02\\Production.dll';
        product= 'D:\\test\\2.00.02\\Production.dll';
        #script = 'D:\\Code\\Production Tool - SVN5470 - PAD_BOX\\Bin\\Script.dll';
        script = 'D:\\test\\2.00.02\\Script.dll';
        #dbg_fw = 'H:\\Firmware\\705A\\lollipop_atm7059a_demo_86v_150120_large_udisk.fw'; 
        dbg_fw = 'D:\\test\\s500\\s500_android_gb5_wxga_150812.fw'; 
    #MakeBin(script, fw, 'AFI', 'f:\\afi.bin');
    #ListPartitions(script, dbg_fw);
    #Production(script, product, 0, 1, dbg_fw);
    para = ProductionV2.BurnPara();

    if Handle_Arg(sys.argv, para) == True :
        print 'path:' + para.fw;
        ProductionV2.Cmd_Production(script, product, dbg_fw, para);


