#!/usr/bin/python
# -*- coding: utf-8 -*-
import os;
import sys;
import time;
import thread;
import random;
import datetime;

from ctypes import *
from string import *
from math   import *

# private module
from CommonV2 import *;
            
# main function          
def MakeCard(script, maker, disk, fw):
    """
    script: firmware parser dll, full path of Script.dll
    maker: card maker dll, full path of CardMaker.dll
    disk: physical disk number, integer, such as 0,1,2...
    """
    card = ClassMaker();
    card.SetDebugMode(disk, fw);
    card.InitMaker(script, maker);
    return card.MakeCard();  

#===============================================================

#type id for Py_ShowMessage
MSG_MSGBOX      = 0;#0x00: 弹消息框; @nSubType=0
#MSG_LOG         = 1;#0x01: 写入日志文件; @nSubType=0
MSG_STATUS      = 2;#0x02: 状态信息; @nSubType=0
MSG_ERROR       = 3;#0x03: 错误/异常信息; @nSubType=0
MSG_PROGRESS    = 4;#0x04: 进度信息; @nSubType=[0..100]

#===============================================================
# card maker class
class ClassMaker():
    def __init__(self):
        self.debug = False;
        if (os.name == 'nt'): # redirect stdout on windows
            self.sysout = sys.stdout;
            out = stdout();
            sys.stdout = out;

        self.err_code = 0;
        self.fwscript   = None;  
        self.maker      = None;
        self.buffer     = (c_byte * CN_PAKET_SIZE)();
        self.boot       = '';
        self.work_dir   = '';
        self.MBR        = None;
        self.part_cap   = [0,0,0,0,0,0,0,0,0,0,0,0]; # MB unit
        self.dbg_fw     = '';
        self.disk       = 0;

        self.timer_start= datetime.datetime.now();

    def __del__(self):
        # Close Firmware
        if (None != self.fwscript):
            self.fwscript.Close();
            
        # release memory
        if (self.buffer != None):
            del self.buffer;
        
        # terminate thread        
        if (None != self.maker):
            self.maker.Py_PyThreadExit(self.err_code);
            if (True == self.debug):
                self.maker.TerminateDbg();

        dt = datetime.datetime.now() - self.timer_start;
        print 'Take time: ', str(dt);        

    def SetDebugMode(self, disk , fw):
        if (disk > 0):
            self.debug = True;  # debug mode for py editor
            sys.stdout = self.sysout;   # revert sys.stdout
            self.dbg_fw= fw;
            self.disk  = disk;

    def EXCEPT(self, ret, show, error, raise_again = True):
        self.err_code = ret;
        print 'EXCEPT: %s, %d\n'%(error, len(error))
        if (len(error) > 0):
            import traceback;
            exp =sys.exc_info();
            if (len(exp) > 2 and exp[2] != None):
                tb = traceback.extract_tb(exp[2])[0];
                fname = str(tb[0]).split(str(os.sep));
                error = error + '(\"%s\" in %s(%s, line %s))'%(tb[3], tb[2], fname[len(fname)-1], tb[1]);
                #print traceback.format_exc();
            
            if (None != self.maker):
                self.maker.Py_ShowMessage(MSG_ERROR, 0, error, len(error));

            if (raise_again == True):
                raise Exception(error);
            
        return ret;

    def SHOW(self, status=None, progress=0):
        if (len(status) > 0):
            self.maker.Py_ShowMessage(MSG_STATUS, 0, status, len(status));
            print '%s(%d)\n'%(status, progress)
            
        if (progress > 0):
            self.maker.Py_ShowMessage(MSG_PROGRESS, progress);
            
    def InitMaker(self, fwParser, maker):
        try:
            self.maker = cdll.LoadLibrary(maker);            
            self.fwscript = FirmwareScript();
            
            if (self.debug == True):
                print 'debug mode, ', self.disk;
                if (os.path.isfile(self.dbg_fw) != True):
                    self.EXCEPT(-2, False, 'Debug firmware non-existent');  
            
                ret = self.maker.StartupDbg(self.dbg_fw, self.disk);
                if (0 == ret):
                    self.EXCEPT(-3, False, 'Failed to startup debugger');

            image = c_buffer('\0', 520);
            fwLen = self.maker.Py_GetFirmwareBin(self.buffer, image);
            if (True != self.fwscript.Open(self.buffer, image, fwParser)):
                self.EXCEPT(-4, False, 'Load firmware failed');

            self.work_dir = '';
            path = str(maker).split(str(os.sep));
            for i in range(len(path)-1):
                self.work_dir += path[i];
                self.work_dir += str(os.sep);

        except Exception as e:
            print str(e);
            self.EXCEPT(-9, False, str(e));
            
        return 0;

    def MakeCard(self):
        try:            
            self.err_code = 0;
            self.SHOW('Opening disk', 0);
            cap = self.maker.Py_OpenDisk();
            print 'Disk Capacity: %d sectors\n'%(cap);         
            if (cap <= 0):
                self.EXCEPT(-19, True, 'Failed to open the disk');

            print 'Disk Capacity: %d sectors\n'%(cap);            
            self.CheckCapacity(cap, -30, 1, 10);

            # MBR
            self.SHOW('Writing MBR');
            memset(addressof(self.buffer), 0, 512);     # sector 0
            self.buffer[510] = 0x55;
            self.buffer[511] = 0xAA;
            if (self.maker.Py_DiskWrite(0, 1, self.buffer) != 1): # write 1 sector data from sector 0
                self.EXCEPT(-49, True, 'Failed to write MBR');
            
            # BOOT
            self.SHOW('Writing BOOT', 11);
            self.BuildBOOT(-50);
            if (self.maker.Py_DiskWrite(1, 0x1FFF, self.buffer) != 0x1FFF): # write 0x1FFF sectors data from sector 1
                self.EXCEPT(-59, True, 'Failed to write BOOT');

            # Logical Partitions
            self.DownloadPartitions(0x2000, cap, 13, 100);# offset and total sectors

            # Update MBR to show udisk
            if (self.MBR != None):
                memset(addressof(self.buffer), 0, 512);     # sector 0
                if (self.maker.Py_DiskRead(0, 1, self.buffer) != 1): # read 1 sector data from sector 0
                    self.EXCEPT(-90, True, 'Failed to read MBR');
                    
                memmove(addressof(self.buffer) + 446, addressof(self.MBR), 16);
                if (self.maker.Py_DiskWrite(0, 1, self.buffer) != 1): # write 1 sector data from sector 0
                    self.EXCEPT(-91, True, 'Failed to update MBR');

        except Exception as e:
            if (self.err_code == 0):
                self.EXCEPT(-99, True, str(e), False);
                
        finally:
            self.maker.Py_CloseDisk();
            self.SHOW('Close disk');
            
        print 'exit_code: %d\n'%(self.err_code)  
        return self.err_code;

    def CheckCapacity(self, diskCap, err, percentS, percentT):
        total = 0;

        # OTA_SHARE
        self.SHOW('Preparing data for card', percentS);
        if (self.maker.Py_PrepareData(percentS, percentT) != 0):
            self.EXCEPT(err, True, 'Failed to prepare data');

        diskCap = diskCap // 2048; #MB

        # PARTITION
        keys = ('PARTITION_F', 'PARTITION_O');
        for key in keys:
            P = self.fwscript.QueryAll(key);
            for i in range(len(P)):
                #(0, u'BOOT', u'./boot.bin', u'RAW', u'IMG', 17)
                p = int(P[i][0]);
                flag = int(P[i][5]);
                size = 0;               

                if( key == 'PARTITION_O'):
                    size = (self.maker.Py_GetImageSize('PARTITION', p, 0) + 2047) // 2048;
                    if (size > 0 and (flag & 0x30) == 0): # ignore
                        size = 16;  # 16M                     
                elif( key == 'PARTITION_F'):
                    size = int(P[i][2]); # MB
                    if ((flag & 0x10) == 0):
                        size = 16; # placeholder 16M

                if ((flag & 0x10) and (size == 0)):
                    self.EXCEPT(err-1, True, '%s missing'%(P[i][1]));

                if (size > 0):
                    self.part_cap[p] = size;
                    total = total + self.part_cap[p];
                    #print P[i], self.part_cap[p], '\n';
        self.boot = '\\mmc-boot.bin';
        
        print 'check capacity mb: %d <> %d\n'%(total, diskCap);
        if (total > diskCap): # MB
            self.EXCEPT(err-2, True, 'Disk capacity is not enough');

        print 'BOOT: %s\n' %(self.boot);

        self.SHOW('', percentT);        
        return True;

    def BuildBOOT(self, err):
        #self.boot='nand-boot.bin';
        nLen = self.maker.Py_GetFileInImage('MISC', self.boot, 0);
        print 'boot.bin size is %x\n'%(nLen);
        if (nLen > 0):
            #self.buffer = (c_byte * nLen)();
            self.maker.Py_GetFileInImage('MISC', self.boot, self.buffer);
        offset = c_uint32(0);
        length = c_uint32(0);
        length.value = nLen;     
        buf = (c_uint8 * 0x100000)();
        memset(addressof(buf), 0, sizeof(buf));
        memmove(addressof(buf), addressof(self.buffer), length.value);  
        
        # merge boot
        MBRC_FISRT_SIZE     = 0x400;
        MBRC_SIZE_OFFSET    = 0x2c;
        MBRC2_SIZE_OFFSET   = 0x28;
        FLASH_INFO_OFFSET   = 0x08;
        AFINFO_OFFSET  = 0x50;
        MBR_INFO_OFFSET     = 0x180; #分区表在afi上的偏移位置
        PARTITION_TBL_SIZE  = 12 * 8;#12个分区
        
        SECTOR_SIZE     = 0x200;#bytes
        PAGE_PER_BLOCK  = 0x100;#1M    #此值偶尔动态变化，若出错，请咨询flash驱动人员
        SECTOR_PER_PAGE = 0x08; #4K
        pageAddrPerBlock = 0x100;
        
        memmove(addressof(offset), addressof(buf) + MBRC2_SIZE_OFFSET, 4);

        mbrc_1st = MBRC_FISRT_SIZE;
        mbrc_2nd = offset.value;
        mbrc_3rd = length.value - mbrc_1st - mbrc_2nd;
        print 'MBRC_SIZE: 0x%x, 0x%x, 0x%x, 0x%x\n'%(mbrc_1st, mbrc_2nd, mbrc_3rd, length.value);

        #startup.bin
        #size = self.fwscript.FileRead('startup.bin', 0, 1, self.buffer, None);
        #off = mbrc_1st + mbrc_2nd - AFINFO_BACK_OFFSET - 0x200;
        #memmove(addressof(buf) + off, addressof(self.buffer), size);

        #afinfo.bin
        
        size = self.fwscript.FileRead('afinfo.bin', 0, 1, self.buffer, None);
        sn_rand=hex(random.randint(0, 0xffffffffffffffff));
        sn_rand=sn_rand.upper();
        memmove(addressof(self.buffer)+0xf0, sn_rand, 16);
        #设置afinfo 0x24位为4；
        #self.buffer[36]=4;
        uart_index = (c_uint8)(3);
        memmove(addressof(self.buffer)+0x24, addressof(uart_index), 1);
        size = self.BuildAFINFO(self.buffer, size); # build afi
        off = mbrc_1st + AFINFO_OFFSET;
        print 'Afinfo Size is %d,off is 0x%x\n'%(size,off); 
        memmove(addressof(buf) + off, addressof(self.buffer), size);

        #mbr_info.bin
        size = self.fwscript.FileRead('mbr_info.bin', 0, 1, self.buffer, None);
        udisk = int(self.fwscript.ParamKey('PARTITION_UDISK_INDEX', 0));
        """
        制作量产卡时，如果data分区=0，设置U盘分区为-1，这样剩余空间给U盘分区.让U盘分区有足够空间保存量产固件
        查partition.cfg及gen_mbr_info.py可知，705A中data分区的索引顺序为4，含第0个，即共5个。
        """
        img_new_size = c_int(0);
        memmove(addressof(img_new_size), addressof(self.buffer) + 5*8+4, 4);
        print 'data_size is %d\n'%(img_new_size.value);
        #if (img_new_size.value == 0): 
        img_new_size.value = -1;
        memmove(addressof(self.buffer) + (udisk+1)*8+4, addressof(img_new_size), 4);

        """
        制作量产卡时，swap 分区清0
        705A中swap分区的索引顺序为9,含第0区为10.
        """
        img_new_size.value = 0;
        memmove(addressof(self.buffer) + 10*8+4, addressof(img_new_size), 4);
        
        for p in range(1, udisk):
            memmove(addressof(self.buffer)+p*8+4, byref(c_uint32(self.part_cap[p])), 4);        
        off = mbrc_1st + AFINFO_OFFSET + MBR_INFO_OFFSET;
        print 'MBR_INFO: 0x%x\n' %(off);
        memmove(addressof(buf) + off, addressof(self.buffer), PARTITION_TBL_SIZE);
        # CapInfo: flash_ptn_1B, partition_num_1B, phy_info_2B, partition_cap_4B(MB)        
        offset.value = 0;
        memmove(addressof(offset), addressof(buf) + off + 10, 2);# rec partition
        memmove(addressof(buf) + FLASH_INFO_OFFSET, byref(c_uint16(pageAddrPerBlock)), 2);
        
        memmove(addressof(buf) + FLASH_INFO_OFFSET + 2, byref(c_uint16(PAGE_PER_BLOCK)), 2);        
        memmove(addressof(buf) + FLASH_INFO_OFFSET + 4, byref(c_uint32(SECTOR_PER_PAGE)), 4);
        print 'Here Is Test3!';
        temp = sizeof(buf);
        print 'buf size is %d'%(temp);
        memmove(addressof(buf) + MBRC_SIZE_OFFSET, byref(c_uint32(mbrc_3rd)), 4);
        print 'Here Is Test5!';
        
        buf[500] = ord('A');
        buf[501] = ord('c');
        buf[502] = ord('t');
        buf[503] = ord('B');
        buf[504] = ord('r');
        buf[505] = ord('m');
        buf[506] = 0xAA;
        buf[507] = 0x55;

        # checksum 1st mbrc
        checksum = checksum_4(buf, 508);
        checksum.value = checksum.value + 0x1234;
        memmove(addressof(buf) + 508, byref(checksum), 4);
 
        # checksum 2nd mbrc
        off = mbrc_1st + mbrc_2nd - 6;
        buf[off]    = 0xAA;
        buf[off+1]  = 0x55;
        off = off + 2;
        checksum = checksum_4(addressof(buf) + mbrc_1st, mbrc_2nd - 4);
        checksum.value = checksum.value + 0x1234;
        memmove(addressof(buf) + off, byref(checksum), 4);

        # check 3rd mbrc
        off = length.value - 6;
        buf[off]    = 0xAA;
        buf[off+1]  = 0x55;
        off = off + 2;
        checksum = checksum_4(addressof(buf) + mbrc_1st + mbrc_2nd, mbrc_3rd - 4);
        checksum.value = checksum.value + 0x1234;
        memmove(addressof(buf) + off, byref(checksum), 4);

        memset(addressof(self.buffer), 0, sizeof(self.buffer));
        memmove(addressof(self.buffer), addressof(buf), 0x100000);
        memmove(addressof(self.buffer) + 0x100000, addressof(buf), 0x100000);
        memmove(addressof(self.buffer) + 0x200000, addressof(buf), 0x100000);
        memmove(addressof(self.buffer) + 0x300000, addressof(buf), 0x100000 - 0x200);
            
    def DownloadPartitions(self, lba, cap, percentS, percentT):
        total = 0;
        for p in range(1, len(self.part_cap)):
            total = total + self.part_cap[p];
        step = (total - (lba // 2048)) / (percentT - percentS); #MB
        #print 'DownloadPartitions: %d .. %d, %d%%-%d%%, %d\n' %(total, lba//2048, percentS, percentT, step);

        parts = int(self.fwscript.ParamKey('PARTITION_UDISK_INDEX', 0)) + 1;        
        PP = [['','','','','','']] * parts;
        
        # PARTITION
        keys = ('PARTITION_F', 'PARTITION_O');
        for key in keys:
            P = self.fwscript.QueryAll(key);
            for i in range(len(P)):
                #(0, u'BOOT', u'./boot.bin', u'RAW', u'IMG', 17)
                PP[int(P[i][0])] = P[i];
                
        for p in range(parts):   
            if(PP[p][4] == 'FMT'):
                print '%s LBA 0x%x, %dMB, %d%%\n' %(PP[p][1].encode('utf-8'), lba, self.part_cap[p], percentS);
                self.SHOW('Formating %s'%(PP[p][1].encode('utf-8')), percentS);
                self.FormatPartition(lba, self.part_cap[p] * 2048); 
                lba = lba + self.part_cap[p] * 2048; # sectors
                percentS = percentS + self.part_cap[p] / step;
                self.SHOW('', percentS);
            
            elif(PP[p][4] == 'IMG'):
                if (len(PP[p][2]) > 0): # PARTITION_I internal
                    self.SHOW('Writing %s (%x)'%(PP[p][1].encode('utf-8'), lba), percentS);
                    while(1):
                        fsize = self.fwscript.FileRead(PP[p][2], 0, 512, self.buffer, None) / 512;
                        if (self.maker.Py_DiskWrite(lba, fsize, self.buffer) != fsize):
                            self.EXCEPT(-80-p, True, 'Failed to write data to the disk!');
                        size = size + fsize;# sectors                    
                        percent_end = percentS + size / 2048 / step;
                        if (percent_end > percentS):
                            self.SHOW(None, percent_end);                            
                        if (fsize * 512 < sizeof(self.buffer)):
                            break;
                    lba = lba + self.part_cap[p] * 2048; # sectors
                    percentS = percentS + self.part_cap[p] / step;
                    self.SHOW('', percentS);
                    continue;
                
                size = self.maker.Py_GetImageSize('PARTITION', int(PP[p][0]), 0);
                if (size == 0):
                    continue;

                print '%s LBA 0x%x, %dMB, %d%%\n'%(PP[p][1].encode('utf-8'), lba, self.part_cap[p], percentS);
                self.SHOW('Writing %s'%(PP[p][1].encode('utf-8')), percentS);                    
                if ((int(PP[p][5]) & 0x30) == 0):# placeholder                    
                    self.FormatPartition(lba, self.part_cap[p] * 2048);                                           
                else:    
                    percent_end = percentS + self.part_cap[p] / step;
                    if (0 != self.maker.Py_DownloadImage('PARTITION', int(PP[p][0]), lba, size, percentS, percent_end)):
                        self.EXCEPT(-80-p, True, 'Failed to write data to the disk!');
                lba = lba + self.part_cap[p] * 2048;
                percentS = percentS + self.part_cap[p] / step;                      
                self.SHOW('', percentS);
                        
            elif (PP[p][4] == 'DIR'):# the last partition
                print '%s LBA 0x%x, %dMB, %d%%\n'%(PP[p][1].encode('utf-8'), lba, self.part_cap[p], percentS);
                self.SHOW('Writing %s'%(PP[p][1].encode('utf-8')), percentS);
                if (0 != self.maker.Py_DownloadImage('PARTITION', int(PP[p][0]), lba, cap-lba, percentS, percentT)):
                    self.EXCEPT(-80-p, True, 'Failed to write data to the disk!');                    
                self.SHOW('', percentT);
                self.BuildMBR(lba, cap-lba);                
                break;
                        
    def FormatPartition(self, lba, sectors):
        memset(addressof(self.buffer), 0xff, len(self.buffer));
        pkt = sizeof(self.buffer) / 512; #sectors
        while(sectors > 0):
            if (sectors >= pkt):
                if (self.maker.Py_DiskWrite(lba, pkt, self.buffer) != pkt):
                    self.EXCEPT(-80, True, 'Failed to write data to the disk!');                
                lba = lba + pkt;
                sectors = sectors - pkt;
            else:
                if (self.maker.Py_DiskWrite(lba, sectors, self.buffer) != sectors):
                    self.EXCEPT(-80, True, 'Failed to write data to the disk!');
                lba = lba + sectors;
                sectors  = 0;

    def BuildMBR(self, lba, cap):
        cfg = c_buffer('\0', 32);
        self.maker.Py_GetConfig('MOUNT', 0, cfg);
        if (cfg.value[0] == '1'):
            # 80 E7 50 14 0B ED 41 DD 00 20 05 00 FF 27 75 00
            self.MBR = (c_uint8 * 16)();
            self.MBR[0] = 0x80;
            self.MBR[4] = 0x0B; # FAT32
            memmove(addressof(self.MBR)+8, byref(c_uint32(lba)), 4);
            memmove(addressof(self.MBR)+12, byref(c_uint32(cap)), 4);
            
            chs = self.LBA2CHS(lba);    # Start CHS
            memmove(addressof(self.MBR)+1, byref(c_uint32(chs)), 3);
            
            lba = lba + cap; 
            chs = self.LBA2CHS(lba);    # end CHS
            memmove(addressof(self.MBR)+5, byref(c_uint32(chs)), 3);

    def BuildAFINFO(self, cbuf, size):
        AFI = ['INF_AFI_DEF_BYTE', 'INF_AFI_DEF_WORD','INF_AFI_DEF_DWORD',\
               'INF_AFI_DEF_BYTE_EXT', 'INF_AFI_DEF_WORD_EXT', 'INF_AFI_DEF_DWORD_EXT'];
        LEN = [1,2,4,1,2,4];        
        for k in range(len(AFI)):
            cfg = self.fwscript.QueryAll(AFI[k]);
            if (cfg == None):
                continue;
            for it in cfg:
                pos = int(it[0]);
                memmove(addressof(cbuf)+pos, byref(c_uint32(int(it[1]))), LEN[k]);
                if (pos > size):
                    size = pos + LEN[k]; 
        size = (size + 511) / 512 * 512;
        return size;
            
    def LBA2CHS(self, lba):        
        TPS = 63;   #每磁道有多少个扇区 Per Track Sector count
        CPH = 255;  #每柱面有多少个磁道 Per Cylinder Header count

        CS = 0; #起始柱面 Cylinder Start
        HS = 0; #起始磁道 Header Start
        SS = 1; #起始扇区 Sector Start
        
        C = lba / (CPH * TPS) + CS;
        H = (lba / TPS) % CPH + HS;
        S = lba % TPS + SS;

        #HSC: H8b,S6b,C10b
        chs = H + S * 0x100 + (C / 0x100) * 0x4000 + (C % 0x100) * 0x10000;
        return chs;

    def CallWinExe(self, argv):
        # args[0]  is the exe file
        import subprocess;
        ret = 0;
        si = subprocess.STARTUPINFO();
        si.dwFlags = subprocess.STARTF_USESHOWWINDOW;
        si.wShowWindow = subprocess.SW_HIDE;
        try:
            handle = subprocess.Popen(
                args = argv,
                shell = True,
                startupinfo = si);
            print 'Running %s,%s, %s\n' %(argv[0], argv[1], argv[2]);
            ret = handle.wait();
            print 'Exit %s\n' %(argv[0]);
        except Exception, e:
            self.EXCEPT(-200, True, 'Failed to run %s!'%(argv[0])); 
            handle  = None

        return ret;        

    #===============================================================
    # 调试参数设置
    def GetDebugConfig(self):
        '''
        Debug paramters
        '''        
        cfg = [];
        # fw should be the first
        cfg.append(('fw', 'H:\\Firmware\\705A\\lollipop_atm7059a_demo_86v_150120_large_udisk.fw'));
        cfg.append(('disk', '2')); 
        return cfg;
    
#================================================================
#'''
if __name__ == '__main__':
    script = 'F:\\CardMaker-rc\\Script.dll';
    maker = 'F:\\CardMaker-rc\\CardMaker.dll';

    import subprocess;
    from inspect import getmembers
    print getmembers(subprocess.STARTUPINFO)

    if (os.name == 'posix'):
        sys.exit(0);     
    elif (os.name == 'nt'):
        script = 'D:\\Code\\Production Tool - SVN5470 - PAD_BOX\\Bin\\Script.dll';
        maker  = 'D:\\Code\\Production Tool - SVN5470 - PAD_BOX\\Bin\\CardMaker.dll';
        dbg_fw = 'H:\\Firmware\\705A\\lollipop_atm7059a_demo_86v_150120_large_udisk.fw';
    try:
        MakeCard(script, maker, 2, dbg_fw);
    except Exception as e:
        print '%s(%s)' %(type(e), e)
    

